import os, sys, time
from sys import platform as _platform
from dropbox import client, rest, session
from conf import APP_KEY, APP_SECRET, ACCESS_TYPE, TARGET_DIR, SYNC_TIME, AUTO_SYNC

BASE_PATH  = os.getcwd()
CHUNK_SIZE = 128 * 1024
TOKEN_FILE = 'db_token.txt'

sess = session.DropboxSession(APP_KEY, APP_SECRET, ACCESS_TYPE)

def get_token():
    try:
        print '\nReading %s' % TOKEN_FILE
        with open(TOKEN_FILE) as f:
            token_key, token_secret = f.read().split('|')
            f.close()
            return token_key, token_secret
    except Exception, e:
        print 'Can not read token file %s - Error %s' % (TOKEN_FILE, e)
        return None, None


def get_client():
    token_key, token_secret = get_token()
    if token_key and token_secret:
        sess.set_token(token_key, token_secret)
        c = client.DropboxClient(sess)
        return c


def init_session():
    request_token = sess.obtain_request_token()
    url = sess.build_authorize_url(request_token)

    print "\nPlease visit this link and press the 'Allow' button, then hit 'Enter' here:\n"
    print url
    
    if _platform == "linux" or _platform == "linux2":
        os.system("sensible-browser " + url)
    elif _platform == "darwin":
        os.system("open " + url)
    elif _platform == "win32":
        os.system("start " + url)
    
    raw_input()

    access_token = sess.obtain_access_token(request_token)
    try:
        token_file = open(TOKEN_FILE, 'w')
        token_file.write('%s|%s' % (access_token.key, access_token.secret))
        token_file.close()
    except Exception, e:
        token_file.close()
        print 'Can not write to file %s - Error %s' % (TOKEN_FILE, e)

    c = client.DropboxClient(sess)
    print "Linked account:\n", c.account_info()

    return c


def recursive_get_folder(client, folder):
    folder_metadata = client.metadata(folder)

    for c in folder_metadata["contents"]:
        if c["is_dir"]:
            dirname = BASE_PATH + "/" + TARGET_DIR + c["path"]
            sys.stdout.write("Found directory %s ... " % dirname)
            sys.stdout.flush()

            if os.path.exists(dirname):
                sys.stdout.write("Exists local.\n")
            else:
                sys.stdout.write("Creating local...")
                sys.stdout.flush()
                os.makedirs(dirname)
                sys.stdout.write("Done.\n")

            sys.stdout.flush()

            previous_cwd = os.getcwd()
            os.chdir(dirname)
            recursive_get_folder(client, c["path"])
            os.chdir(previous_cwd)
        else:
            # Check if the file exists
            download = True
            local_filename = BASE_PATH + "/" + TARGET_DIR + c["path"]
            sys.stdout.write("Checking %s ... " % c["path"])
            sys.stdout.flush()

            if os.path.exists(local_filename):
                sys.stdout.write("Exists ")
                sys.stdout.flush()
                # Compare if they are the same
                if os.path.getsize(local_filename) == c["bytes"]:

                # FIXME: Improve reliability
                #if os.path.getmtime(filename) == c.client_mtime:

                    sys.stdout.write("and are the same.\n")
                    sys.stdout.flush()
                    download = False
                else:
                    sys.stdout.write("but aren't the same. ")
            else:
                sys.stdout.write("Does not exist. ")
                sys.stdout.flush()

            if download:
                # Download the file
                sys.stdout.write("\nDownloading.. ")
                sys.stdout.flush()

                out = open(local_filename, "wb")
                f, metadata = client.get_file_and_metadata(c["path"])
                total_size = metadata["bytes"]
                written = 0
                while True:
                    data = f.read(CHUNK_SIZE)
                    if data == '':
                        break
                    else:
                        out.write(data)
                        written = written + len(data)

                    percent = written * 30 / total_size
                    percent_line = "[%s%s] %d/%d" % (("*" * percent),
                                                     (" " * (30 - percent)),
                                                     written, total_size)

                    sys.stderr.write(percent_line)
                    sys.stderr.write("\b" * (len(percent_line)))
                    sys.stderr.flush()

                out.close()

                sys.stdout.write("Done. \n")


if __name__ == "__main__":

    if not os.path.exists(TARGET_DIR):
        os.makedirs(TARGET_DIR)

    if not os.path.exists(BASE_PATH + "/" + TOKEN_FILE):
        init_session()

    client = get_client()

    if AUTO_SYNC:
        print 'Start watching, saved to: %s' % TARGET_DIR
        time_loop = 1
        try:
            while True:
                time.sleep(1)
                time_loop += 1
                if not time_loop % SYNC_TIME:
                    print 'Auto sync every %s seconds' % SYNC_TIME
                    recursive_get_folder(client, "/")
                    os.system("stabe co")
        except KeyboardInterrupt:
            print 'End watching.'
    else:
        print "Start to downloading...\n"
        recursive_get_folder(client, "/")